!Jonathan Eggleston Sept 2013, modified the file csv_write to write to latex table fragment instead

!Jonathan Eggleston April 2013, I got ride of the include statement in this file, which allows this module to be compiled by only using latex_file.f90

! latex_file.f90 --
!  This was written by Arjen Markus
!     Small module to facilitate writing CSV-files
!
!     $Id: latex_file.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
!     The module contains the following subroutines:
!     latex_next_record       Advance to the next record
!     latex_write_integer     Write a single integer to the CSV-file
!     latex_write_real        Write a single real to the CSV-file
!     latex_write_dble        Write a single double-precision real to the
!                           CSV-file
!     latex_write_char        Write a single character string to the
!                           CSV-file
!     latex_write_integer_1d  Write a 1D array of integers to the CSV-file
!     latex_write_real_1d     Write a 1D array of reals to the CSV-file
!     latex_write_dble_1d     Write a 1D array of double-precision real to
!                           the CSV-file
!     latex_write_char_1d     Write a 1D array of character strings to the
!                           CSV-file
!     latex_write_integer_2d  Write a 2D array of integers to the CSV-file
!     latex_write_real_2d     Write a 2D array of reals to the CSV-file
!     latex_write_dble_2d     Write a 2D array of double-precision real to
!                           the CSV-file
!     latex_write_char_2d     Write a 2D array of character strings to the
!                           CSV-file
!
!     For convenience, the generic name "latex_write" can be used
!     instead of the individual routines.
!
!     The file to write to must already be opened as a LU-number
!     is passed.
!
!     Layout of the CSV-file:
!     - single items are written to the end of the current record
!     - one-dimensional items are also written to the end of the current
!       record
!     - two-dimensional items are written to separate records, one for
!       each row
!     - except for the two-dimensional versions, all routines allow
!       you to suppress advancing to the next record:
!       - for single items you must indicate whether to advance or not
!       - for one-dimensional items, the argument is optional. Default
!         is to advance.
!
!     Note on the format:
!     CSV-files apparently come in different guises (Kernighan and Pike,
!     The practice of Programming, Addison-Wesley, 1999). This module
!     uses the following rules:
!     - items are always separated by a single comma (,)
!     - string items are delimited by double quotes (")
!     - embedded double quotes are treated by doubling the quote
!     - trailing blanks are considered irrelevant
!
module latex_file
    implicit none
    interface latex_write
        module procedure latex_write_integer
        module procedure latex_write_integer_1d
        module procedure latex_write_integer_2d
        module procedure latex_write_char
        module procedure latex_write_char_1d
        module procedure latex_write_char_2d
        module procedure latex_write_real
        module procedure latex_write_real_1d
        module procedure latex_write_real_2d
        module procedure latex_write_dble
        module procedure latex_write_dble_1d
        module procedure latex_write_dble_2d
    end interface
contains

! latex_next_record --
!     Go to the next record (convenience routine)
! Arguments:
!     lun        LU-number of the CSV-file
! Result:
!     The current record is closed, the next write will be to the
!     new record
! Note:
!     This is a convenience routine, it may result in a superfluous
!     comma at the end of the previous record. This does not seem to
!     be problematic, though, for MS Excel
!
subroutine latex_next_record( lun )
    integer, intent(in)   :: lun

    write(lun,'(a)') ' \\'
end subroutine latex_next_record

!JE.  Following subroutine takes the buffer from the write integer, real, or character subroutines and then finally writes them to a file

subroutine latex_write_support( lun, buffer_in, advance, emp )
    integer, intent(in)   :: lun
    character(len=*), intent(in)     :: buffer_in
    logical, intent(in)   :: advance
    character(len=1), intent(in), optional :: emp

    character(len=15 + len(buffer_in) )     :: buffer
    if (present(emp) ) then

        select case( emp)
            case( 'b' )
                buffer = '\textbf{' // trim(adjustl(buffer_in)) // '}'

            case( 'i' )
                buffer = '\textit{' // trim(adjustl(buffer_in)) // '}'

            case default

                print *, "error in select case for emp in latex_write_support, stopping"
                stop

        end select

    else

        buffer = trim(adjustl(buffer_in))

    end if

    if ( advance ) then
       write(lun,'(a,a)') trim(buffer), ' \\'
    else
       ! Most probably: write the comma only when needed
       ! - depends on other actions
       write(lun,'(a,a)',advance='no') trim(buffer), ' & '
    endif

end subroutine latex_write_support


! latex_write_integer/real/dble --
!     Write a single integer/real/double precision real to the CSV-file
! Arguments:
!     lun        LU-number of the CSV-file
!     value      Value to write
!     advance    Advance (.true.) or not, so that more items can be
!                written to the same record
! Result:
!     The value is written to the current record of the CSV-file
!






subroutine latex_write_integer( lun, value, advance, emp )
    integer, intent(in)   :: lun
    integer, intent(in)   :: value
    logical, intent(in)   :: advance
    character(len=1), intent(in), optional :: emp

    character(len=40)     :: buffer
    write( buffer, '(I10)' ) value

    if( present(emp) ) then
        call latex_write_support( lun, buffer, advance, emp )
    else
        call latex_write_support( lun, buffer, advance )
    end if


end subroutine latex_write_integer

subroutine latex_write_real( lun, value, advance, emp )
    integer, intent(in)   :: lun
    real, intent(in)      :: value
    logical, intent(in)   :: advance
    character(len=1), intent(in), optional :: emp

    character(len=40)     :: buffer
    write( buffer, '(G14.6)' ) value

    if( present(emp) ) then
        call latex_write_support( lun, buffer, advance, emp )
    else
        call latex_write_support( lun, buffer, advance )
    end if


end subroutine latex_write_real

subroutine latex_write_dble( lun, value, advance, emp )
    integer, intent(in)                    :: lun
    real(kind=kind(1.0d0)), intent(in)     :: value
    logical, intent(in)                    :: advance
    character(len=1), intent(in), optional :: emp

    character(len=40)     :: buffer
    write( buffer, '(G20.12)' ) value

    if( present(emp) ) then
        call latex_write_support( lun, buffer, advance, emp )
    else
        call latex_write_support( lun, buffer, advance )
    end if

end subroutine latex_write_dble

! latex_write_char --
!     Write a single character string to the CSV-file
! Arguments:
!     lun        LU-number of the CSV-file
!     value      Value to write
!     advance    Advance (.true.) or not, so that more items can be
!                written to the same record
! Result:
!     The value is written to the current record of the CSV-file
!
subroutine latex_write_char( lun, value, advance, emp )
    integer, intent(in)            :: lun
    character(len=*), intent(in)   :: value
    logical, intent(in)            :: advance
    character(len=1), intent(in), optional :: emp

    integer                        :: k
    integer                        :: pos
    integer                        :: posb
    character(len=2*len(value))    :: buffer

    buffer = value

    !
    ! Check for nasty characters (")
    !
    k    = index( value,'"')
    pos  = 1
    posb = 1
    do while ( k .ge. 1 )
        buffer(posb:)   = value(pos:)
        buffer(posb+k:) = '"' // value(pos+k:)
        pos             = pos  + k + 1
        posb            = posb + k + 2
        k               = index( value(pos:),'"')
    enddo

    if( present(emp) ) then
        call latex_write_support( lun, buffer, advance, emp )
    else
        call latex_write_support( lun, buffer, advance )
    end if

end subroutine latex_write_char

! latex_write_integer/real/dble_1d --
!     Write a one-dimensional array of items to the CSV-file
! Arguments:
!     lun        LU-number of the CSV-file
!     array      Array to write
!     advance    Advance (.true.) or not, so that more items can be
!                written to the same record
! Result:
!     The array is written to the current record of the CSV-file
! Note:
!     Because the four routines of this type differ only in
!     the data type, we use an include file for the body.
!
subroutine latex_write_integer_1d( lun, array, advance, emp )
    integer, dimension(:), intent(in)   :: array
! cvs_file_1d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the one-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_1d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    logical, intent(in), optional       :: advance
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.
    if ( present(advance) ) adv = advance


    if (present(emp) ) then

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false., emp )
        enddo
        call latex_write( lun, array(size(array)), adv, emp )

    else

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false. )
        enddo
        call latex_write( lun, array(size(array)), adv )

    end if


! end of body

end subroutine latex_write_integer_1d

subroutine latex_write_real_1d( lun, array, advance, emp )
    real, dimension(:), intent(in)   :: array

! cvs_file_1d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the one-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_1d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    logical, intent(in), optional       :: advance
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.
    if ( present(advance) ) adv = advance

    if (present(emp) ) then

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false., emp )
        enddo
        call latex_write( lun, array(size(array)), adv, emp )

    else

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false. )
        enddo
        call latex_write( lun, array(size(array)), adv )

    end if
!
! end of body

end subroutine latex_write_real_1d

subroutine latex_write_dble_1d( lun, array, advance, emp )
    real(kind=kind(1.0d0)), dimension(:), intent(in)   :: array

! cvs_file_1d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the one-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_1d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    logical, intent(in), optional       :: advance
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.
    if ( present(advance) ) adv = advance

    if (present(emp) ) then

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false., emp )
        enddo
        call latex_write( lun, array(size(array)), adv, emp )

    else

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false. )
        enddo
        call latex_write( lun, array(size(array)), adv )

    end if
!
! end of body

end subroutine latex_write_dble_1d

subroutine latex_write_char_1d( lun, array, advance, emp )
    character(len=*), dimension(:), intent(in)   :: array

! cvs_file_1d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the one-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_1d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    logical, intent(in), optional       :: advance
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.
    if ( present(advance) ) adv = advance

    if (present(emp) ) then

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false., emp )
        enddo
        call latex_write( lun, array(size(array)), adv, emp )

    else

        do i = 1,size(array)-1
            call latex_write( lun, array(i), .false. )
        enddo
        call latex_write( lun, array(size(array)), adv )

    end if
!
! end of body

end subroutine latex_write_char_1d

! latex_write_integer/real/dble_2d --
!     Write a two-dimensional array of items to the CSV-file
! Arguments:
!     lun        LU-number of the CSV-file
!     array      Array to write
! Result:
!     The array is written to the current record of the CSV-file
! Note:
!     Because the four routines of this type differ only in
!     the data type, we use an include file for the body.
!
subroutine latex_write_integer_2d( lun, array, emp )
    integer, dimension(:,:), intent(in)   :: array

! cvs_file_2d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the two-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_2d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.

    if (present(emp) ) then

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    else

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    end if
!
! end of body

end subroutine latex_write_integer_2d

subroutine latex_write_real_2d( lun, array, emp )
    real, dimension(:,:), intent(in)   :: array

! cvs_file_2d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the two-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_2d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.

    if (present(emp) ) then

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    else

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    end if
!
! end of body

end subroutine latex_write_real_2d

subroutine latex_write_dble_2d( lun, array, emp )
    real(kind=kind(1.0d0)), dimension(:,:), intent(in)   :: array

! cvs_file_2d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the two-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_2d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.

    if (present(emp) ) then

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    else

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    end if
!
! end of body

end subroutine latex_write_dble_2d

subroutine latex_write_char_2d( lun, array, emp )
    character(len=*), dimension(:,:), intent(in)   :: array

! cvs_file_2d.f90 --
!     Include file for latex_file.f90:
!     contains the body of the two-dimensional version of the
!     writing routines.
!
!     $Id: latex_file_2d.f90,v 1.2 2006/03/26 19:03:53 arjenmarkus Exp $
!
    integer, intent(in)                 :: lun
    character(len=1), intent(in), optional :: emp

    logical                             :: adv
    integer                             :: i

    adv = .true.

    if (present(emp) ) then

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    else

        do i = 1,size(array,2)
            call latex_write( lun, array(:,i), adv )
        enddo

    end if
!
! end of body

end subroutine latex_write_char_2d

end module latex_file
