
module sim_path_mod
   !module for simulating outcomes for households, used to construct my policy experiments


use enum_mod
use model_params_mod
use interp_support_mod

use utility_function_mod
use model_probs_mod
use logit_mod

use iterate_object_mod
use value_grid_object_mod
use iterate_mod

use normal_mod
use stat_estimates_mod

use likelihood_mod
use bayesian_mod

use csv_file
use mpi_mod

   integer, parameter :: n_sim_uniform = 9
   integer, parameter :: n_sim_normal = 5
   integer, parameter :: n_sim_ev = 4
   integer, parameter :: sample_label_length = 100 !max lenght for the sampel labels in my simulation exercize.

   real*8, dimension(n_hetero), parameter :: mean_zero = (/0,0/)

   logical, parameter :: sim_error_check = .false. !controls whether to check for errors
   logical, parameter :: sim_print = .false. !controls whether to print extra output

   real*8, parameter :: ln_m_wage_mean_data = 3.118101
   real*8, parameter :: ln_m_wage_sd_data = .5487114
   real*8, parameter :: ln_wage_mean_data = 3.118101
   real*8, parameter :: ln_wage_sd_data = .5487114


contains




subroutine gen_mygrid_init_sim( n_obs_in, sipp_filename_in, params_in, mygrid_out, &
   m_job_in_opt, job_in_opt)
   !generates initial vector of variables based on household's I observe in the data

   integer, intent(in) :: n_obs_in
   character(len=30), intent(in) :: sipp_filename_in
   real*8, dimension(n_params), intent(in) :: params_in
   type(mygrid_type), dimension( n_obs_in ) , intent(out) :: mygrid_out
   integer, intent(in), optional :: m_job_in_opt, job_in_opt

   type(data_i_type), dimension(:), allocatable  :: sipp_data
   integer :: i, j
   real*8, dimension(n_hetero) :: hetero_vec_temp
   real*8, dimension(n_hetero, n_hetero ) :: hetero_chol_temp

   !first get the sipp data
   call read_sipp_data( sipp_filename_in, sipp_data )

   !get the cholesky decomp for the unobserved heterogeneity
   hetero_chol_temp = 0
   hetero_chol_temp(1,1) = params_in(param_hetero_m_wage_var)
   hetero_chol_temp(2,1) = params_in(param_hetero_wage_covar)
   hetero_chol_temp(2,2) = params_in(param_hetero_wage_var)



   do j = 1, n_obs_in

      !get random household
      i = nint( r_uniform( dble(min_obs), dble(max_obs) ) )
      !now get random household with a particular m_job, job combination, if desired
      if ( present( m_job_in_opt) .and. present( job_in_opt ) ) then
         call get_job_combo( i, m_job_in_opt, job_in_opt )

      end if

      !draw their unosberved heterogeneity
      call get_random_mvn( n_hetero, mean_zero, hetero_chol_temp, hetero_vec_temp )


      !now setup the mygrid object based on this randomly selected household
      call mygrid_out(j) % allocate()

      mygrid_out(j) % rv(1:2) = &
         wage_mean_support_func (  sipp_data(i), hetero_vec_temp(1:2), params_in )


      mygrid_out(j) % iv( var_m_job )           = sipp_data(i) % data_t(1) % m_job
      mygrid_out(j) % iv( var_job )             = sipp_data(i) % data_t(1) % job
      mygrid_out(j) % iv( var_m_hours )         = sipp_data(i) % data_t(1) % m_hours
      mygrid_out(j) % iv( var_hours )           = sipp_data(i) % data_t(1) % hours


      mygrid_out(j) % rv( var_ur_state )        = sipp_data(i) % data_t(1) % ur_state
      mygrid_out(j) % rv( var_uimax )           = sipp_data(i) % data_t(1) % uimax

      mygrid_out(j) % rv( var_uilength )        = &
         ui_length_func(   sipp_data(i) % data_t(1) % ur_state, &
                           sipp_data(i) % data_t(1) % eb_opt_trig, &
                           sipp_data(i) % data_t(1) % euc_bill_1, &
                           sipp_data(i) % data_t(1) % euc_bill_2, &
                           sipp_data(i) % data_t(1) % euc_bill_3 )

      mygrid_out(j) % rv( var_kids )            = sipp_data(i) % data_t(1) % kids
      mygrid_out(j) % rv( var_youngest )        = sipp_data(i) % data_t(1) % youngest

      !now do somethign sepcial for assets
      mygrid_out(j) % rv( var_assets )          = asset_support_func( sipp_data(i) )

      mygrid_out(j) % rv( var_m_wage )          = sipp_data(i) % data_t(1) % m_wage
      mygrid_out(j) % rv( var_wage )            = sipp_data(i) % data_t(1) % wage
      mygrid_out(j) % rv( var_m_non_pec_norm )  = rnorm() * mygrid_out(j) % rv( var_m_job )
      mygrid_out(j) % rv( var_non_pec_norm )    = rnorm() * mygrid_out(j) % rv( var_job )
      mygrid_out(j) % rv( var_m_ui_ben )        = sipp_data(i) % m_ui_ben_init
      mygrid_out(j) % rv( var_ui_ben )          = sipp_data(i) % ui_ben_init

      mygrid_out(j) % rv( var_m_job_length )    = sipp_data(i) % m_job_length_start
      mygrid_out(j) % rv( var_job_length )      = sipp_data(i) % job_length_start
      mygrid_out(j) % rv( var_m_unemp_length )  = (   sipp_data(i) % m_min_unemp_length + &
                                                      sipp_data(i) % m_max_unemp_length ) /2
      mygrid_out(j) % rv( var_unemp_length )    = (   sipp_data(i) % min_unemp_length + &
                                                      sipp_data(i) % max_unemp_length ) /2

!      call print_data_i(sipp_data(i) )
!      call print_data_t(sipp_data(i) % data_t(1))


!      call mygrid_out(j) % print( .true., .true. )

!      stop


      !finally, put mygrid between the right bounds
      call mygrid_bounds_all (mygrid_out(j) )


      if (sim_error_check .eqv. .true. ) then
         call mygrid_out(j) % mygrid_error_hunt(.true., .true. ,j,1,1)

      end if

   end do

   !don't think this is necessary, but doing it anyway just to make sure the sipp_data is removed
   call sipp_data_deallocate( sipp_data )

contains


function wage_mean_support_func( hh_data_in, hetero_in, params_in ) result(wage_mean)
   !gets the wage means
   real*8, dimension(2) :: wage_mean
   type(data_i_type), intent(in) :: hh_data_in
   real*8, dimension(2) :: hetero_in
   real*8, dimension(n_params) :: params_in


   !now get wage means
   wage_mean(1) = &
      params_in(param_wage_cons)  + &
      params_in(param_wage_age)          * hh_data_in  % m_age + &
      params_in(param_wage_age2)         * hh_data_in  % m_age * hh_data_in  % m_age + &
      params_in(param_wage_race)         * hh_data_in  % m_white + &
      params_in(param_wage_hs)           * hh_data_in  % m_hs + &
      params_in(param_wage_some_college) * hh_data_in  % m_some_college + &
      params_in(param_wage_college)      * hh_data_in  % m_college + &
      hetero_in(1)


   wage_mean(2) = &
      params_in(param_wage_cons)  + &
      params_in(param_wage_age)          * hh_data_in  % age + &
      params_in(param_wage_age2)         * hh_data_in  % age * hh_data_in  % age + &
      params_in(param_wage_race)         * hh_data_in  % white + &
      params_in(param_wage_hs)           * hh_data_in  % hs + &
      params_in(param_wage_some_college) * hh_data_in  % some_college + &
      params_in(param_wage_college)      * hh_data_in  % college + &
      params_in(param_wage_gender) + &
      hetero_in(2)



end function wage_mean_support_func


function asset_support_func( hh_data_in) result(assets)
   !sets assets equal to this household's average asset value

   real*8 :: assets
   type(data_i_type), intent(in) :: hh_data_in

   integer :: counter

   counter = 0
   assets = 0

   if (( hh_data_in % liquid_time1 > 0 ) .and. ( hh_data_in % liquid_time1 <= hh_data_in % max_time )) then

      assets = assets + hh_data_in % liquid_net1
      counter = counter + 1

   end if

   if (( hh_data_in % liquid_time2 > 0 ) .and. ( hh_data_in % liquid_time2 <= hh_data_in % max_time )) then

      assets = assets + hh_data_in % liquid_net2
      counter = counter + 1

   end if

   if (( hh_data_in % liquid_time3 > 0 ) .and. ( hh_data_in % liquid_time3 <= hh_data_in % max_time )) then

      assets = assets + hh_data_in % liquid_net3
      counter = counter + 1

   end if

   if ( counter .eq. 0 ) then

      assets = r_uniform(  grid_min( var_assets) , &
                           grid_mean( var_assets ) + grid_sd(var_assets)*.25 )
   else

      assets = assets/counter

   end if


end function asset_support_func

subroutine get_job_combo(index_inout, m_job_in, job_in )

   integer, intent(in out) :: index_inout
   integer, intent(in) :: m_job_in, job_in


   logical :: index_found
   integer :: j

   index_found = .false.
   j = index_inout

   do while ( index_found .eqv. .false. )

      if ( ( sipp_data(j) % data_t(1) % m_job .eq. m_job_in ) .and. &
           ( sipp_data(j) % data_t(1) % job .eq. job_in ) ) then

         index_found = .true.

      else

         if ( j .eq. max_obs ) then
            j = min_obs
         else
            j = j + 1
         end if

         if ( j .eq. index_inout ) then
            print *, "loop through all the observations and didn't find an appropriate m_job, &
               job combination, stopping"
            call mpi_abort(mpi_comm_world, 911, err)
         end if

      end if

   end do

   index_inout = j


   !makes sure the index is for the job combo of interest

end subroutine get_job_combo


end subroutine gen_mygrid_init_sim

subroutine adjust_init_m_jobloss(n_obs_in, mygrid_inout, rv_adj_logic_in, rv_adj_val_in )
   !adjust the randomly drawn mygrid_init for the policy experiments where the husband starts out unemployed and the wife hasn't been
      !working

   integer, intent(in) :: n_obs_in
   type(mygrid_type), dimension( n_obs_in ) , intent(in out) :: mygrid_inout
   logical, dimension(n_cont_vars) :: rv_adj_logic_in
   real*8, dimension( n_cont_vars) :: rv_adj_val_in

   integer :: i, j, m_job, job
   real*8 :: ln_m_wage_mean_sim, ln_m_wage_sd_sim
   real*8, dimension(n_obs_in) :: m_wage_mean_vec
   type(mygrid_type) ::  mygrid_for_m_ui


   !first get the wage distribution means and sd
      !this will be node specific, but I don't care too much about that
   do i=1,n_obs_in
      m_wage_mean_vec(i) = mygrid_inout(i) % rv( var_m_wage_mean )
   end do

   ln_m_wage_mean_sim = mean_scalar_est( n_obs_in, m_wage_mean_vec )

   if ( n_obs_in > 1 ) then
      ln_m_wage_sd_sim = sd_scalar_est( n_obs_in, m_wage_mean_vec, ln_m_wage_mean_sim )
   else
      ln_m_wage_sd_sim = 1
   end if

   call mygrid_for_m_ui % allocate()




   do i=1,n_obs_in

      m_job = mygrid_inout(i) % iv( var_m_job )
      job   = mygrid_inout(i) % iv( var_job )

      do j=1,n_cont_vars
         if (( var_present(j, m_job, job ) .eq. 1) .and. ( rv_adj_logic_in(j) .eqv. .true. )) then
            mygrid_inout(i) % rv(j) = rv_adj_val_in(j)

         end if
      end do



      if ( m_job .eq. 1 ) then
         mygrid_for_m_ui % rv( var_m_wage ) = mygrid_inout(i) % rv( var_m_wage)
         mygrid_for_m_ui % iv( var_m_hours ) = mygrid_inout(i) % iv( var_m_hours)
      else
         mygrid_for_m_ui % rv( var_m_wage ) = &
            exp( normal_adjust_func(   mygrid_inout(i) % rv( var_m_wage_mean ), &
                                       ln_m_wage_mean_sim, &
                                       ln_m_wage_sd_sim, &
                                       ln_m_wage_mean_data, &
                                       ln_m_wage_sd_data ) )

         mygrid_for_m_ui % iv( var_m_hours ) = 40

      end if

      mygrid_for_m_ui % rv( var_uimax ) = mygrid_inout(i) % rv( var_uimax )


      !now make this household unemployed with only the husband receiving ui

      !first husband
      mygrid_inout(i) % rv( var_m_ui_ben )         = m_ui_ben_func( mygrid_for_m_ui , 1 )

      mygrid_inout(i) % iv( var_m_job )            = 0

      mygrid_inout(i) % rv( var_m_wage )           = 0
      mygrid_inout(i) % iv( var_m_hours )          = 0
      mygrid_inout(i) % rv( var_m_non_pec_norm )   = 0
      mygrid_inout(i) % rv( var_m_job_length )     = 0

      if ( rv_adj_logic_in( var_m_unemp_length ) .eqv. .true. ) then
         mygrid_inout(i) % rv( var_m_unemp_length )   = rv_adj_val_in( var_m_unemp_length )

      else
         mygrid_inout(i) % rv( var_m_unemp_length )   = 0
      end if




      mygrid_inout(i) % rv( var_ui_ben ) = ui_ben_func( mygrid_inout(i) , 1 )

      mygrid_inout(i) % iv( var_job )        = 0

      mygrid_inout(i) % rv( var_wage )           = 0
      mygrid_inout(i) % iv( var_hours )          = 0
      mygrid_inout(i) % rv( var_non_pec_norm )   = 0
      mygrid_inout(i) % rv( var_job_length )     = 0

      if ( rv_adj_logic_in( var_unemp_length ) .eqv. .true. ) then
         mygrid_inout(i) % rv( var_unemp_length )   = rv_adj_val_in( var_unemp_length )

      else
         mygrid_inout(i) % rv( var_unemp_length )   = grid_max( var_unemp_length)

      end if



   end do




contains

function normal_adjust_func(x_in, mu, sigma, mu_prime, sigma_prime )
   !given a normal with mean mu and sd sigma, transforms this normal into a normal with mean mu_prime and sd sigma_prime
   real*8 :: x_in, mu, sigma, mu_prime, sigma_prime, normal_adjust_func

   normal_adjust_func = mu_prime + sigma_prime* ( (x_in - mu)/sigma)

end function normal_adjust_func

end subroutine adjust_init_m_jobloss



subroutine adjust_init_sim( n_obs_in, mygrid_inout, rv_adj_logic_in, rv_adj_val_in, &
   m_job_post_in, job_post_in, m_ui_post_in, ui_post_in )
   !adjust the randomly drawn mygrid_init for the policy experiments where I want household's to have certain characteristics

   integer, intent(in) :: n_obs_in
   type(mygrid_type), dimension( n_obs_in ) , intent(in out) :: mygrid_inout
   logical, dimension(n_cont_vars) :: rv_adj_logic_in
   real*8, dimension( n_cont_vars) :: rv_adj_val_in
   integer, intent(in) :: m_job_post_in, job_post_in, m_ui_post_in, ui_post_in !what value to adjust the input values to

   integer :: i, j, m_job_pre, job_pre
   real*8 :: ln_m_wage_mean_sim, ln_m_wage_sd_sim, ln_wage_mean_sim, ln_wage_sd_sim
   real*8, dimension(n_obs_in) :: m_wage_mean_vec, wage_mean_vec
   type(mygrid_type) ::  mygrid_temp
   real*8 :: ui_temp


   !first get the wage distribution means and sd
      !this will be node specific, but I don't care too much about that
   do i=1,n_obs_in
      m_wage_mean_vec(i) = mygrid_inout(i) % rv( var_m_wage_mean )
      wage_mean_vec(i)   = mygrid_inout(i) % rv( var_wage_mean )

   end do

   ln_m_wage_mean_sim = mean_scalar_est( n_obs_in, m_wage_mean_vec )
   ln_wage_mean_sim = mean_scalar_est( n_obs_in, wage_mean_vec )

   if ( n_obs_in > 1 ) then
      ln_m_wage_sd_sim = sd_scalar_est( n_obs_in, m_wage_mean_vec, ln_m_wage_mean_sim )
      ln_wage_sd_sim = sd_scalar_est( n_obs_in, wage_mean_vec, ln_wage_mean_sim )
   else
      ln_m_wage_sd_sim = 1
      ln_wage_sd_sim = 1
   end if

   !make sure sd isn't too small (important if all init's have the same value)
   ln_m_wage_sd_sim = max(  ln_m_wage_sd_sim, zero_tol )
   ln_wage_sd_sim = max(  ln_wage_sd_sim, zero_tol )




   call mygrid_temp % allocate()




   do i=1,n_obs_in


      m_job_pre = mygrid_inout(i) % iv( var_m_job )
      job_pre   = mygrid_inout(i) % iv( var_job )


      !get wages and ui benefits, which are either taken directly from the inputs or are gotten through
         !the wage means

      !first husband wages
      if ( m_job_pre .eq. 1 ) then
         mygrid_temp % rv( var_m_wage )         = mygrid_inout(i) % rv( var_m_wage)
         mygrid_temp % iv( var_m_hours )        = mygrid_inout(i) % iv( var_m_hours)
         mygrid_temp % rv( var_m_non_pec_norm ) = mygrid_inout(i) % rv( var_m_non_pec_norm )
         mygrid_temp % rv( var_m_job_length )   = mygrid_inout(i) % rv( var_m_job_length )

      else
         mygrid_temp % rv( var_m_wage ) = &
            exp( normal_adjust_func(   mygrid_inout(i) % rv( var_m_wage_mean ), &
                                       ln_m_wage_mean_sim, &
                                       ln_m_wage_sd_sim, &
                                       ln_m_wage_mean_data, &
                                       ln_m_wage_sd_data ) )

         mygrid_temp % iv( var_m_hours )        = 40
         mygrid_temp % rv( var_m_non_pec_norm ) = 0
         mygrid_temp % rv( var_m_job_length )   = 0

      end if

      !now wife's wages
      if ( job_pre .eq. 1 ) then
         mygrid_temp % rv( var_wage )           = mygrid_inout(i) % rv( var_wage)
         mygrid_temp % iv( var_hours )          = mygrid_inout(i) % iv( var_hours)
         mygrid_temp % rv( var_non_pec_norm )   = mygrid_inout(i) % rv( var_non_pec_norm )
         mygrid_temp % rv( var_job_length )     = mygrid_inout(i) % rv( var_job_length )

      else
         mygrid_temp % rv( var_wage ) = &
            exp( normal_adjust_func(   mygrid_inout(i) % rv( var_wage_mean ), &
                                       ln_wage_mean_sim, &
                                       ln_wage_sd_sim, &
                                       ln_wage_mean_data, &
                                       ln_wage_sd_data ) )

         mygrid_temp % iv( var_hours )          = 40
         mygrid_temp % rv( var_non_pec_norm )   = 0
         mygrid_temp % rv( var_job_length )     = 0

      end if



      if ( rv_adj_logic_in( var_uimax ) .eqv. .true. ) then
         mygrid_inout(i) % rv( var_uimax ) = rv_adj_val_in( var_uimax )
         mygrid_temp % rv( var_uimax )     = rv_adj_val_in( var_uimax )
      else
         mygrid_temp % rv( var_uimax )   = mygrid_inout(i) % rv( var_uimax )
      end if


      !now get the unemployment variables

      !first husband
      ui_temp = m_ui_ben_func( mygrid_temp , m_ui_post_in )
      mygrid_temp % rv( var_m_ui_ben ) = ui_temp

      if ( m_job_pre .eq. 0 ) then
         mygrid_temp % rv( var_m_unemp_length ) = mygrid_inout(i) % rv( var_m_unemp_length )
      else
         mygrid_temp % rv( var_m_unemp_length ) = 0
      end if

      !now wife
      ui_temp = ui_ben_func( mygrid_temp , ui_post_in )
      mygrid_temp % rv( var_ui_ben ) = ui_temp

      if ( job_pre .eq. 0 ) then
         mygrid_temp % rv( var_unemp_length ) = mygrid_inout(i) % rv( var_unemp_length )
      else
         mygrid_temp % rv( var_unemp_length ) = 0
      end if


      !now finally, transform the mygrid_inout
      mygrid_inout(i) % iv( var_m_job ) = m_job_post_in
      mygrid_inout(i) % iv( var_job ) = job_post_in

      !first husband
      if ( m_job_post_in .eq. 1 ) then
         mygrid_inout(i) % rv( var_m_wage )           = mygrid_temp % rv( var_m_wage )
         mygrid_inout(i) % iv( var_m_hours )          = mygrid_temp % iv( var_m_hours )
         mygrid_inout(i) % rv( var_m_non_pec_norm )   = mygrid_temp % rv( var_m_non_pec_norm )
         mygrid_inout(i) % rv( var_m_job_length )     = mygrid_temp % rv( var_m_job_length )

         mygrid_inout(i) % rv( var_m_ui_ben )         = 0
         mygrid_inout(i) % rv( var_m_unemp_length )   = 0

      else
         mygrid_inout(i) % rv( var_m_wage )           = 0
         mygrid_inout(i) % iv( var_m_hours )          = 0
         mygrid_inout(i) % rv( var_m_non_pec_norm )   = 0
         mygrid_inout(i) % rv( var_m_job_length )     = 0

         mygrid_inout(i) % rv( var_m_ui_ben )         = mygrid_temp % rv( var_m_ui_ben )
         mygrid_inout(i) % rv( var_m_unemp_length )   = mygrid_temp % rv( var_m_unemp_length )

      end if

      !now wife
      if ( job_post_in .eq. 1 ) then
         mygrid_inout(i) % rv( var_wage )           = mygrid_temp % rv( var_wage )
         mygrid_inout(i) % iv( var_hours )          = mygrid_temp % iv( var_hours )
         mygrid_inout(i) % rv( var_non_pec_norm )   = mygrid_temp % rv( var_non_pec_norm )
         mygrid_inout(i) % rv( var_job_length )     = mygrid_temp % rv( var_job_length )

         mygrid_inout(i) % rv( var_ui_ben )         = 0
         mygrid_inout(i) % rv( var_unemp_length )   = 0

      else
         mygrid_inout(i) % rv( var_wage )           = 0
         mygrid_inout(i) % iv( var_hours )          = 0
         mygrid_inout(i) % rv( var_non_pec_norm )   = 0
         mygrid_inout(i) % rv( var_job_length )     = 0

         mygrid_inout(i) % rv( var_ui_ben )         = mygrid_temp % rv( var_ui_ben )
         mygrid_inout(i) % rv( var_unemp_length )   = mygrid_temp % rv( var_unemp_length )

      end if


      !now adjust any other variables that need to be adjusted
      do j=1,n_cont_vars
         if (( var_present(j, m_job_post_in, job_post_in ) .eq. 1) .and. &
               ( rv_adj_logic_in(j) .eqv. .true. )) then
            mygrid_inout(i) % rv(j) = rv_adj_val_in(j)

         end if
      end do


   end do

   !check for errors if I need to
   if (sim_error_check .eqv. .true. ) then

      do i = 1, n_obs_in
         call mygrid_inout(i) % mygrid_error_hunt( .true., .true., i, 1, 1)
      end do

   end if




contains

function normal_adjust_func(x_in, mu, sigma, mu_prime, sigma_prime )
   !given a normal with mean mu and sd sigma, transforms this normal into a normal with mean mu_prime and sd sigma_prime
   real*8 :: x_in, mu, sigma, mu_prime, sigma_prime, normal_adjust_func

   normal_adjust_func = mu_prime + sigma_prime* ( (x_in - mu)/sigma)

end function normal_adjust_func

end subroutine adjust_init_sim



subroutine gen_rand_vars_sim( n_obs_in, max_t_in, rand_uniform_out, rand_normal_out, rand_ev_out )
   !generates the random variables for the simulation routine

   integer, intent(in) :: n_obs_in
   integer, intent(in) :: max_t_in
   real*8, dimension(n_sim_uniform, max_t_in, n_obs_in) :: rand_uniform_out
   real*8, dimension(n_sim_normal, max_t_in, n_obs_in) :: rand_normal_out
   real*8, dimension(n_sim_ev, max_t_in, n_obs_in) :: rand_ev_out

   integer :: i, j, t

!   print *, "n_obs_in", n_obs_in
!   print *, "max_t_in", max_t_in
!   print *, "n_sim_normal", n_sim_normal

   call random_number( rand_uniform_out )

   do i = 1, n_obs_in
      do t = 1, max_t_in
         do j =1, n_sim_normal

            rand_normal_out(j,t,i) = rnorm()

         end do
      end do
   end do

   do i = 1, n_obs_in
      do t = 1, max_t_in
         do j =1, n_sim_ev

            rand_ev_out(j,t,i) = rand_ev_func()

         end do
      end do
   end do



end subroutine

subroutine gen_rand_vars_sim_1hh( max_t_in, rand_uniform_out, rand_normal_out, rand_ev_out )
   !generates the random variables for the simulation routine (for one household)

   integer, intent(in) :: max_t_in
   real*8, dimension(n_sim_uniform, max_t_in) :: rand_uniform_out
   real*8, dimension(n_sim_normal, max_t_in) :: rand_normal_out
   real*8, dimension(n_sim_ev, max_t_in) :: rand_ev_out

   integer :: i, j, t

 !  print *, "max_t_in", max_t_in
 !  print *, "n_sim_normal", n_sim_normal

   call random_number( rand_uniform_out )

      do t = 1, max_t_in
         do j =1, n_sim_normal

            rand_normal_out(j,t) = rnorm()

         end do
      end do

      do t = 1, max_t_in
         do j =1, n_sim_ev

            rand_ev_out(j,t) = rand_ev_func()

         end do
      end do



end subroutine


subroutine sim_one_t(mygrid_in, params_in, rand_uniform_in, rand_normal_in, rand_ev_in, value_matrix_in, &
   asset_matrix_in, max_switch_in, mygrid_out )
   !simulates path of random variables
   type(mygrid_type), intent(in) :: mygrid_in
   real*8, dimension(n_params) :: params_in
   real*8, dimension(n_sim_uniform) :: rand_uniform_in
   real*8, dimension(n_sim_normal) :: rand_normal_in
   real*8, dimension(n_sim_ev) :: rand_ev_in
   real*8, dimension(n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension(n_grid_points ), intent(in) :: asset_matrix_in
   logical, intent(in) :: max_switch_in !declares whether to max the value function wrt assets or to calculate assets with the interpolation function
   type(mygrid_type), intent(out) :: mygrid_out

   type(mygrid_type) :: mygrid_both_reject, mygrid_husband_accept, mygrid_wife_accept, &
      mygrid_both_accept
   real*8, dimension(n_utility_params) :: utility_params
   real*8, dimension(4) :: values_vec
   real*8 :: assets_temp, values_temp
   integer :: birth, m_receive_new, receive_new
   integer :: opt_choice

   !set up some of the mygrid objects
   call mygrid_out % allocate()
   call mygrid_both_reject % allocate()
   mygrid_both_reject = mygrid_in




   !not dealing with birth for now since I think that would create wierd stationary distribution issues

!   !now deal with births
!   if ( rand_uniform_in(iter_birth) < birth_prob_iter_func(mygrid_in, params_in)  ) then
!      mygrid_both_reject % rv( var_kids )       = min(mygrid_in % rv( var_kids ) +1 , grid_max( var_kids ) )
!      mygrid_both_reject % rv( var_youngest )   = 0
!   else
!      mygrid_both_reject % rv( var_youngest )   = min( mygrid_in % rv( var_youngest ) + age_increment, &
!         grid_max( var_youngest) )
!   end if

   !deal with husband losing his job
   if (mygrid_in % iv(var_m_job) .eq. 1) then

      if ( rand_uniform_in(iter_m_jobloss) < m_joboffer_prob_iter_func(mygrid_in, params_in) ) then

         !first determine whether ui is received
         if ( rand_uniform_in(iter_m_ui) < m_ui_prob_iter_func(mygrid_in, params_in) ) then
            mygrid_both_reject % rv( var_m_ui_ben ) = m_ui_ben_func(mygrid_in, 1)

            mygrid_both_reject % rv( var_m_ui_ben ) = &
               bounds_func(   mygrid_both_reject % rv( var_m_ui_ben ), &
                              grid_min(var_m_ui_ben), &
                              grid_max(var_m_ui_ben ) )

         else
            mygrid_both_reject % rv( var_m_ui_ben ) = 0
         end if

         !now set the job variable to 0
         mygrid_both_reject % iv( var_m_job )            = 0
         mygrid_both_reject % rv( var_m_wage )           = 0
         mygrid_both_reject % iv( var_m_hours )          = 0
         mygrid_both_reject % rv( var_m_non_pec_norm )   = 0
         mygrid_both_reject % rv( var_m_job_length )     = 0


      end if !of whther the husband lost his job
   end if !of whther husband has a job

   !deal with wife losing his job
   if (mygrid_in % iv(var_job) .eq. 1) then

      !deal with the husband loosing his job
      if ( rand_uniform_in(iter_jobloss) < joboffer_prob_iter_func(mygrid_in, params_in) ) then

         !first determine whether ui is received
         if ( rand_uniform_in(iter_ui) < ui_prob_iter_func(mygrid_in, params_in) ) then
            mygrid_both_reject % rv( var_ui_ben ) = ui_ben_func(mygrid_in, 1)

            mygrid_both_reject % rv( var_ui_ben ) = &
               bounds_func(   mygrid_both_reject % rv( var_ui_ben ), &
                              grid_min(var_ui_ben), &
                              grid_max(var_ui_ben ) )
         else
            mygrid_both_reject % rv( var_ui_ben ) = 0
         end if

         !now set the job variable to 0
         mygrid_both_reject % iv( var_job )           = 0
         mygrid_both_reject % rv( var_wage )          = 0
         mygrid_both_reject % iv( var_hours )         = 0
         mygrid_both_reject % rv( var_non_pec_norm )  = 0
         mygrid_both_reject % rv( var_job_length )    = 0

      end if !of whther the husband lost his job
   end if !of whther husband has a job

   !now change unemployment rate
   mygrid_both_reject % rv( var_ur_state ) = params_in(param_state_ur_cons) + &
      params_in(param_state_ur_on_state_ur) * mygrid_in % rv( var_ur_state ) + &
      params_in(param_state_ur_sd) * rand_normal_in( iter_norm_ur )

   !make sure ur doesn't go past bounds
   mygrid_both_reject % rv(var_ur_state) = bounds_func( &
      mygrid_both_reject % rv(var_ur_state) , &
      grid_min( var_ur_state), grid_max( var_ur_state) )



   !increment the appropriate length variables
   if (mygrid_both_reject % iv( var_m_job ) .eq. 1 ) then
      mygrid_both_reject % rv( var_m_job_length ) = &
      min(  mygrid_both_reject % rv( var_m_job_length ) + 1, grid_max( var_m_job_length) )
   else
      mygrid_both_reject % rv( var_m_unemp_length ) = &
      min(  mygrid_both_reject % rv( var_m_unemp_length ) + 1, grid_max( var_m_unemp_length) )
   end if


   if (mygrid_both_reject % iv( var_job ) .eq. 1 ) then
      mygrid_both_reject % rv( var_job_length ) = &
      min(  mygrid_both_reject % rv( var_job_length ) + 1, grid_max( var_job_length) )
   else
      mygrid_both_reject % rv( var_unemp_length ) = &
      min(  mygrid_both_reject % rv( var_unemp_length ) + 1, grid_max( var_unemp_length)  )
   end if

   !set ui to 0 if have exceded max duration
   if (  nint(mygrid_both_reject % rv(var_m_unemp_length) ) > &
      nint(mygrid_both_reject % rv(var_uilength) ) ) then

         mygrid_both_reject % rv(var_m_ui_ben) = 0
   end if

   if (  nint(mygrid_both_reject % rv(var_unemp_length) ) > &
      nint(mygrid_both_reject % rv(var_uilength) ) ) then

         mygrid_both_reject % rv(var_ui_ben) = 0
   end if




   !now see if husband gets a job offer, and if so, get the job variables
   if ( rand_uniform_in(iter_m_job_offer) < m_joboffer_prob_iter_func(mygrid_in, params_in) ) then
    !  print *, "m got offer"
      call mygrid_husband_accept % allocate()
      mygrid_husband_accept = mygrid_both_reject

      m_receive_new = 1
      mygrid_husband_accept % iv( var_m_job ) = 1

      !hours
      if ( rand_uniform_in(iter_m_hours) < params_in(param_prob_pt) ) then
         mygrid_husband_accept % iv( var_m_hours ) = 20
      else
         mygrid_husband_accept % iv( var_m_hours ) = 40
      end if

      mygrid_husband_accept % rv( var_m_wage)  = &
         m_wage_new_func ( mygrid_husband_accept, &
                           params_in, &
                           rand_normal_in ( iter_norm_m_wage) )

      !non pec
      mygrid_husband_accept % rv( var_m_non_pec_norm ) = rand_normal_in ( iter_norm_m_non_pec )

      !put wage and non-pecuniary between bounds
      mygrid_husband_accept % rv( var_m_wage) = &
         bounds_func(   mygrid_husband_accept % rv( var_m_wage), &
                        grid_min( var_m_wage), &
                        grid_max( var_m_wage) )

      mygrid_husband_accept % rv( var_m_non_pec_norm) = &
         bounds_func(   mygrid_husband_accept % rv( var_m_non_pec_norm), &
                        grid_min( var_m_non_pec_norm), &
                        grid_max( var_m_non_pec_norm) )

      !set husband's non-employment variables to 0
      mygrid_husband_accept % rv( var_m_ui_ben)  = 0
      mygrid_husband_accept % rv( var_m_unemp_length)  = 0


   else
      m_receive_new = 0
   end if !for whether husband receives a job offer

   !now see if wife gets a job offer, and if so, get the job variables
   if ( rand_uniform_in(iter_job_offer) < joboffer_prob_iter_func(mygrid_in, params_in) ) then
    !  print *, "m got offer"

      call mygrid_wife_accept % allocate()
      mygrid_wife_accept = mygrid_both_reject

      receive_new = 1
      mygrid_wife_accept % iv( var_job ) = 1

      !hours
      if ( rand_uniform_in(iter_hours) < params_in(param_prob_pt) ) then
         mygrid_wife_accept % iv( var_hours ) = 20
      else
         mygrid_wife_accept % iv( var_hours ) = 40
      end if

      mygrid_wife_accept % rv( var_wage)  = &
         wage_new_func (   mygrid_wife_accept, &
                           params_in, &
                           rand_normal_in ( iter_norm_wage) )

      !non pec
      mygrid_wife_accept % rv( var_non_pec_norm ) = rand_normal_in ( iter_norm_non_pec )

      !put wage and non-pecuniary between bounds
      mygrid_wife_accept % rv( var_wage) = &
         bounds_func(   mygrid_wife_accept % rv( var_wage), &
                        grid_min( var_wage), &
                        grid_max( var_wage) )

      mygrid_wife_accept % rv( var_non_pec_norm) = &
         bounds_func(   mygrid_wife_accept % rv( var_non_pec_norm), &
                        grid_min( var_non_pec_norm), &
                        grid_max( var_non_pec_norm) )

      !set wife's non-employment variables to 0
      mygrid_wife_accept % rv( var_ui_ben)  = 0
      mygrid_wife_accept % rv( var_unemp_length)  = 0

   else
      receive_new = 0
   end if !for whether husband receives a job offer


   !now generate the mygrid if both have received a job offer
   if ( ( m_receive_new .eq. 1 ) .and. ( receive_new .eq. 1 ) ) then
      call mygrid_both_accept % allocate()
      mygrid_both_accept = mygrid_husband_accept

      !now get the wife's variables
      mygrid_both_accept % iv( var_job )           = 1
      mygrid_both_accept % iv( var_hours )         = mygrid_wife_accept % iv( var_hours )
      mygrid_both_accept % rv( var_wage )          = mygrid_wife_accept % rv( var_wage )
      mygrid_both_accept % rv( var_non_pec_norm )  = mygrid_wife_accept % rv( var_non_pec_norm )

      !set wife's non-employment variables to 0
      mygrid_both_accept % rv( var_ui_ben)         = 0
      mygrid_both_accept % rv( var_unemp_length)   = 0

   end if

   !setup utility parameters
   call utility_function_setup(  mygrid_both_reject % rv(var_kids), &
                                 mygrid_both_reject % rv(var_youngest), &
                                 params_in, &
                                 utility_params )





   !now get the assets and values+utility for each choice

   !first set the values_vec to -huge, makes latter max problem easier to deal with
   values_vec = -huge( values_vec )


   if( max_switch_in .eqv. .true. ) then !gets assest and values from maximizing the value function

      !both reject
      call max_value_sub( mygrid_both_reject, utility_params, params_in, value_matrix_in, &
         assets_temp, values_temp )

      mygrid_both_reject % rv( var_assets ) = assets_temp
      values_vec( index_both_reject) = values_temp + rand_ev_in( index_both_reject )





      !only husband accepts
      if ( m_receive_new .eq. 1 ) then
         call max_value_sub( mygrid_husband_accept, utility_params, params_in, value_matrix_in, &
            assets_temp, values_temp )

         mygrid_husband_accept % rv( var_assets ) = assets_temp
         values_vec( index_husband_accept ) = values_temp + rand_ev_in( index_husband_accept )
      end if

      !only wife accept
      if ( receive_new .eq. 1 ) then
         call max_value_sub( mygrid_wife_accept, utility_params, params_in, value_matrix_in, &
            assets_temp, values_temp )

         mygrid_wife_accept % rv( var_assets ) = assets_temp
         values_vec( index_wife_accept ) = values_temp + rand_ev_in( index_wife_accept )
      end if

      !both accept
      if ( ( m_receive_new .eq. 1 ) .and. ( receive_new .eq. 1 ) ) then
         call max_value_sub( mygrid_both_accept, utility_params, params_in, value_matrix_in, &
            assets_temp, values_temp )

         mygrid_both_accept % rv( var_assets ) = assets_temp
         values_vec( index_both_accept ) = values_temp + rand_ev_in( index_both_accept )
      end if

   else

     ! print*, "assets", mygrid_both_reject % rv( var_assets )


      !both reject
      call nomax_value_sub( mygrid_both_reject, utility_params, params_in, value_matrix_in, &
         asset_matrix_in, assets_temp, values_temp )

      mygrid_both_reject % rv( var_assets ) = assets_temp

     ! print*, "assets", mygrid_both_reject % rv( var_assets )
     ! stop

      values_vec( index_both_reject) = values_temp + rand_ev_in( index_both_reject )

      !only husband accepts
      if ( m_receive_new .eq. 1 ) then
         call nomax_value_sub( mygrid_husband_accept, utility_params, params_in, value_matrix_in, &
            asset_matrix_in, assets_temp, values_temp )

         mygrid_husband_accept % rv( var_assets ) = assets_temp
         values_vec( index_husband_accept ) = values_temp + rand_ev_in( index_husband_accept )
      end if

      !only wife accept
      if ( receive_new .eq. 1 ) then
         call nomax_value_sub( mygrid_wife_accept, utility_params, params_in, value_matrix_in, &
            asset_matrix_in, assets_temp, values_temp )

         mygrid_wife_accept % rv( var_assets ) = assets_temp
         values_vec( index_wife_accept ) = values_temp + rand_ev_in( index_wife_accept )
      end if

      !both accept
      if ( ( m_receive_new .eq. 1 ) .and. ( receive_new .eq. 1 ) ) then
         call nomax_value_sub( mygrid_both_accept, utility_params, params_in, value_matrix_in, &
            asset_matrix_in, assets_temp, values_temp )

         mygrid_both_accept % rv( var_assets ) = assets_temp
         values_vec( index_both_accept ) = values_temp + rand_ev_in( index_both_accept )
      end if

   end if

   !now assign mygrid out to the mygrid which provides the highest utility
   opt_choice = maxloc( values_vec, 1 )

   if ( opt_choice .eq. index_both_reject ) then

      mygrid_out = mygrid_both_reject

   else if ( opt_choice .eq. index_husband_accept ) then

      mygrid_out = mygrid_husband_accept

   else if ( opt_choice .eq. index_wife_accept ) then

      mygrid_out = mygrid_wife_accept

   else

      mygrid_out = mygrid_both_accept

   end if



   if (sim_print .eqv. .true. ) then
      !this section of my code checks the variables
      print *, "    "
      print *, "mygrid both reject"
      call mygrid_both_reject % print( .true., .true. )
      call mygrid_both_reject % mygrid_error_hunt( .true., .true., 1,2,1 )

      if ( m_receive_new .eq. 1 ) then
         print *, "    "
         print *, "mygrid husband accept"
         call mygrid_husband_accept % print( .true., .true. )
         call mygrid_husband_accept % mygrid_error_hunt( .true., .true., 1,2,1 )
      end if


      if( receive_new .eq. 1 ) then
         print *, "    "
         print *, "mygrid wife accept"
         call mygrid_wife_accept % print( .true., .true. )
         call mygrid_wife_accept % mygrid_error_hunt( .true., .true., 1,2,1 )
      end if


      if ( ( m_receive_new .eq. 1) .and. ( receive_new .eq. 1 ) ) then
         print *, "    "
         print *, "mygrid both accept"
         call mygrid_both_accept % print( .true., .true. )
         call mygrid_both_accept % mygrid_error_hunt( .true., .true., 1,2,1 )
      end if


      print *, " "
      print *, "value vector"
      print *, "both reject   ", values_vec( index_both_reject )
      print *, "husband accept", values_vec( index_husband_accept )
      print *, "wife accept   ", values_vec( index_wife_accept )
      print *, "both accept   ", values_vec( index_both_accept )



      print *, "optimal choice"
      print *, opt_choice


      print *, "choice prob"
      print *, mlogit_prob( 4, opt_choice, values_vec - rand_ev_in )

   !   print *, "rand ev in ", rand_ev_in

   !   read(*,*)

   end if

   if ( (sim_print .eqv. .false. ) .and. (sim_error_check .eqv. .true. ) ) then

      call mygrid_both_reject % mygrid_error_hunt( .true., .true., 1,2,1 )

      if ( m_receive_new .eq. 1 ) then
         call mygrid_husband_accept % mygrid_error_hunt( .true., .true., 1,2,2 )
      end if

      if( receive_new .eq. 1 ) then
         call mygrid_wife_accept % mygrid_error_hunt( .true., .true., 1,2,3 )
      end if

      if ( ( m_receive_new .eq. 1) .and. ( receive_new .eq. 1 ) ) then
         call mygrid_both_accept % mygrid_error_hunt( .true., .true., 1,2,4 )
      end if


   end if




contains


subroutine nomax_value_sub(mygrid_in, utility_params_in, params_in, value_matrix_in, &
   asset_matrix_in, asset_choice_out, value_out )

 !does same thing as max_value_sub expect that it generate assets through interpolation rather than maximization

   type( mygrid_type ), intent(in) :: mygrid_in
   real*8, dimension(n_utility_params), intent(in) :: utility_params_in
   real*8, dimension(n_params), intent(in) :: params_in
   real*8, dimension( n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension( n_grid_points ), intent(in) :: asset_matrix_in
   real*8, intent(out) :: asset_choice_out
   real*8, intent(out) :: value_out

   real*8 :: util

   asset_choice_out = asset_policy_func(mygrid_in, asset_matrix_in )

   util = utility_function( mygrid_in, asset_choice_out, utility_params_in, params_in)

   value_out = util + value_function( mygrid_in, asset_choice_out, value_matrix_in )


end subroutine


end subroutine sim_one_t



subroutine sim_path (n_obs_in, max_t_in, n_burn_in, mygrid_init, params_in, rand_uniform_in, &
   rand_normal_in, rand_ev_in, value_matrix_in, asset_matrix_in, max_switch_in, mygrid_out )

   !simulates a path of variables for a sample of households
   integer, intent(in) :: n_obs_in
   integer, intent(in) :: max_t_in
   integer, intent(in) :: n_burn_in
   type(mygrid_type), dimension(n_obs_in), intent(in) :: mygrid_init
   real*8, dimension(n_params) :: params_in

   real*8, dimension(n_sim_uniform, max_t_in, n_obs_in) :: rand_uniform_in
   real*8, dimension(n_sim_normal, max_t_in, n_obs_in) :: rand_normal_in
   real*8, dimension(n_sim_ev, max_t_in, n_obs_in) :: rand_ev_in

   real*8, dimension(n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension(n_grid_points ), intent(in) :: asset_matrix_in

   logical, intent(in) :: max_switch_in !declares whether to max the value function wrt assets or to calculate assets with the interpolation function
   type(mygrid_type), dimension( max_t_in - n_burn_in, n_obs_in), intent(out) :: mygrid_out

   type(mygrid_type) :: mygrid_old, mygrid_new
   integer :: i,t

   call mygrid_old % allocate()
 !  call mygrid_new % allocate()


   do i = 1, n_obs_in

      mygrid_old = mygrid_init(i)

      do t=1, max_t_in

       !  print *, "i, t", i, t

         call sim_one_t( mygrid_old, params_in, rand_uniform_in(:,t,i), rand_normal_in(:,t,i), &
            rand_ev_in(:,t,i), value_matrix_in, asset_matrix_in, max_switch_in, mygrid_new )

         mygrid_old = mygrid_new

         if (t > n_burn_in ) mygrid_out( t - n_burn_in, i ) = mygrid_new

      end do

   end do


end subroutine sim_path



subroutine sim_path_1hh ( max_t_in, n_burn_in, mygrid_init, params_in, rand_uniform_in, &
   rand_normal_in, rand_ev_in, value_matrix_in, asset_matrix_in, max_switch_in, mygrid_out )

   !simulates a path of variables for a sample of households
   integer, intent(in) :: max_t_in
   integer, intent(in) :: n_burn_in
   type(mygrid_type), intent(in) :: mygrid_init
   real*8, dimension(n_params) :: params_in

   real*8, dimension(n_sim_uniform, max_t_in ) :: rand_uniform_in
   real*8, dimension(n_sim_normal, max_t_in ) :: rand_normal_in
   real*8, dimension(n_sim_ev, max_t_in ) :: rand_ev_in

   real*8, dimension(n_grid_points ), intent(in) :: value_matrix_in
   real*8, dimension(n_grid_points ), intent(in) :: asset_matrix_in

   logical, intent(in) :: max_switch_in !declares whether to max the value function wrt assets or to calculate assets with the interpolation function
   type(mygrid_type), dimension( max_t_in - n_burn_in ), intent(out) :: mygrid_out

   type(mygrid_type) :: mygrid_old, mygrid_new
   integer :: t

   call mygrid_old % allocate()
 !  call mygrid_new % allocate()



      mygrid_old = mygrid_init

      do t=1, max_t_in

       !  print *, "i, t", i, t

         call sim_one_t( mygrid_old, params_in, rand_uniform_in(:,t), rand_normal_in(:,t), &
            rand_ev_in(:,t), value_matrix_in, asset_matrix_in, max_switch_in, mygrid_new )

         mygrid_old = mygrid_new

         if (t > n_burn_in ) mygrid_out( t - n_burn_in ) = mygrid_new

      end do




end subroutine sim_path_1hh



subroutine sim_path_write_init(filename_in )
   !open a file and writes the header for the output from simulating paths
   character(len=30), intent(in) :: filename_in

   integer :: k, unit_int

   open(newunit= unit_int, file=filename_in, status = "replace")


   call gen_var_labels()

   !now write the var labels for the header

   call csv_write( unit_int, "sample", .false. )
   call csv_write( unit_int, "hh", .false. )
   call csv_write( unit_int, "time", .false. )

   call csv_write( unit_int, "m_job", .false. )
   call csv_write( unit_int, "job", .false. )
   call csv_write( unit_int, "m_hours", .false. )
   call csv_write( unit_int, "hours", .false. )

   do k = 1, n_cont_vars - 1
      call csv_write( unit_int, var_labels(k), .false. )
   end do

   call csv_write( unit_int, var_labels(n_cont_vars), .true. )

   close(unit_int)


end subroutine sim_path_write_init



subroutine sim_path_write( filename_in, sample_num_in, n_obs_in, n_t_in, mygrid_in )
   !use csv_write to write the simulated path to a file
   character(len=30), intent(in) :: filename_in
   integer, intent(in) :: sample_num_in !for distinguishing samples (e.g. 1= base case, 2=noui )
   integer, intent(in) :: n_obs_in
   integer, intent(in) :: n_t_in
   type(mygrid_type), dimension(n_t_in,n_obs_in), intent(in) ::mygrid_in

   integer :: i, obs_no_base, proc_no, unit_int

   obs_no_base = 0
   call mpi_barrier(mpi_comm_world, err)

   !now make sure only one process is writing to the file at once
   do proc_no = 0, nprocs -1

      if ( myrank .eq. proc_no ) then

         open(newunit = unit_int, file = filename_in, position = "append" )

         do i = 1,n_obs_in

            call sim_path_write_inner( unit_int, &
                                       sample_num_in, &
                                       i+ obs_no_base, &
                                       n_t_in, &
                                       mygrid_in(:,i) )

         end do

         close(unit_int)

      end if


      obs_no_base = obs_no_base + n_obs_in

      call mpi_barrier(mpi_comm_world, err)

   end do


contains

subroutine sim_path_write_inner( unit_int_in, sample_num_in, obs_no_in, n_t_in, mygrid_in )
   !use csv_write to write the simulated path to a file for one household
   integer, intent(in) :: unit_int_in
   integer, intent(in) :: sample_num_in !for distinguishing samples (e.g. 1= base case, 2=noui )
   integer, intent(in) :: obs_no_in !observation number for this household
   integer, intent(in) :: n_t_in
   type(mygrid_type), dimension(n_t_in), intent(in) ::mygrid_in

   integer :: k,t

   do t=1, n_t_in

      call csv_write( unit_int_in, sample_num_in, .false. )
      call csv_write( unit_int_in, obs_no_in, .false. )
      call csv_write( unit_int_in, t, .false. )

      do k = 1, n_int_vars
         call csv_write( unit_int_in, mygrid_in(t) % iv(k), .false. )
      end do

      do k = 1, n_cont_vars - 1
         call csv_write( unit_int_in, mygrid_in(t) % rv(k), .false. )
      end do

      call csv_write( unit_int_in, mygrid_in(t) % rv(n_cont_vars), .true. )

   end do !over t

end subroutine sim_path_write_inner


end subroutine sim_path_write


subroutine sim_path_job_freq_write( n_samples_in, n_obs_in, n_t_in, mygrid_in, sample_labels_in, &
   filename_in )
   !writes the frequency for each job combination to a file
   integer, intent(in) :: n_samples_in, n_obs_in, n_t_in
   type(mygrid_type), dimension(n_t_in, n_obs_in, n_samples_in)  :: mygrid_in
   character(len=30), dimension(n_samples_in), intent(in) :: sample_labels_in
   character(len=30), intent(in) :: filename_in

   integer, dimension(4, n_samples_in ) :: job_count_matrix
   real*8, dimension(4, n_samples_in ) :: job_freq_matrix_temp, job_freq_matrix_root
   integer :: i,s,t, m_job, job, unit_int

   job_count_matrix = 0
   job_freq_matrix_temp = 0
   job_freq_matrix_root = 0

   !first count the total number of observation for each (m_job, job) combo
   do s=1, n_samples_in
   do i=1, n_obs_in
   do t=1,n_t_in

      m_job = mygrid_in(t,i,s) % iv( var_m_job )
      job   = mygrid_in(t,i,s) % iv( var_job )

      if ( (m_job .eq. 1) .and. ( job .eq. 1) ) then
         job_count_matrix( both_job, s )     = job_count_matrix( both_job, s ) + 1

      else if ( m_job .eq. 1) then
         job_count_matrix( husband_job, s )  = job_count_matrix( husband_job, s ) + 1

      else if ( job .eq. 1 ) then
         job_count_matrix( wife_job, s )     = job_count_matrix( wife_job, s ) + 1

      else
         job_count_matrix( both_nojob, s )   = job_count_matrix( both_nojob, s ) + 1

      end if

   end do
   end do
   end do

   !now get the frequencies for this process
   job_freq_matrix_temp = dble( job_count_matrix ) / dble( n_obs_in * n_t_in )

   !now get the frequencies across processes
   call mpi_reduce( job_freq_matrix_temp, job_freq_matrix_root, size(job_freq_matrix_temp), &
      mpi_real8, mpi_sum, 0, mpi_comm_world, err)




   !now have the root process write to a file
   if ( myrank .eq. 0 ) then

      !divide my number of processes
      job_freq_matrix_root = job_freq_matrix_root / nprocs !updated sept 19 2013

      do s = 1, n_samples_in
         do i =1 , 4
            call round_sub( job_freq_matrix_root( i, s ), 4 )
         end do
      end do

      open(newunit = unit_int, file = filename_in, status = "replace" )

         call csv_write( unit_int, "Sample", .false. )
         call csv_write( unit_int, "Both with Job", .false. )
         call csv_write( unit_int, "Husband with Job", .false. )
         call csv_write( unit_int, "Wife with Job", .false. )
         call csv_write( unit_int, "Both without Job", .true. )

         do s = 1, n_samples_in
            call csv_write( unit_int, sample_labels_in(s), .false. )
            call csv_write( unit_int, job_freq_matrix_root(:,s), .true. )

         end do

      close( unit_int )

   end if

end subroutine sim_path_job_freq_write



subroutine gen_job_freq( n_obs_in, n_t_in, mygrid_in, job_freq_vec_out)
   !gets the sample frequencies for each m_job, job category

   integer, intent(in) :: n_obs_in, n_t_in
   type(mygrid_type), dimension(n_t_in, n_obs_in )  :: mygrid_in
   real*8, dimension(4) :: job_freq_vec_out

   integer, dimension(4) :: job_count_vec
   real*8, dimension(4) :: job_freq_vec_myproc
   integer :: i,t, m_job, job, unit_int

   job_count_vec = 0
   job_freq_vec_myproc = 0
   job_freq_vec_out = 0

   !first count the total number of observation for each (m_job, job) combo
   do i=1, n_obs_in
   do t=1,n_t_in

      m_job = mygrid_in(t,i) % iv( var_m_job )
      job   = mygrid_in(t,i) % iv( var_job )

      if ( (m_job .eq. 1) .and. ( job .eq. 1) ) then
         job_count_vec( both_job )     = job_count_vec( both_job ) + 1

      else if ( m_job .eq. 1) then
         job_count_vec( husband_job )  = job_count_vec( husband_job ) + 1

      else if ( job .eq. 1 ) then
         job_count_vec( wife_job )     = job_count_vec( wife_job ) + 1

      else
         job_count_vec( both_nojob )   = job_count_vec( both_nojob ) + 1

      end if

   end do
   end do

   !now get the frequencies for this process
   job_freq_vec_myproc = dble( job_count_vec ) / dble( n_obs_in * n_t_in )

   !now get the frequencies across processes
   call mpi_allreduce( job_freq_vec_myproc, job_freq_vec_out, size(job_freq_vec_myproc), &
      mpi_real8, mpi_sum, mpi_comm_world, err)

   !and divide my number of processes
   job_freq_vec_out = job_freq_vec_out / nprocs


end subroutine gen_job_freq



subroutine job_freq_write( n_samples_in, job_freq_matrix_in, sample_labels_in, filename_in )

   !writes the frequency for each job combination to a file
   integer, intent(in) :: n_samples_in
   real*8, dimension( 4, n_samples_in ) :: job_freq_matrix_in
   character(len=sample_label_length), dimension(n_samples_in), intent(in) :: sample_labels_in
   character(len=30), intent(in) :: filename_in

   real*8, dimension(4, n_samples_in ) :: job_freq_matrix
   integer :: i,s,t, m_job, job, unit_int

   !now have the root process write to a file
   if ( myrank .eq. 0 ) then

      job_freq_matrix = job_freq_matrix_in

      do s = 1, n_samples_in
         do i =1 , 4
            call round_sub( job_freq_matrix( i, s ), 4 )
         end do
      end do

      open(newunit = unit_int, file = filename_in, status = "replace" )

         call csv_write( unit_int, "Sample", .false. )
         call csv_write( unit_int, "Both with Job", .false. )
         call csv_write( unit_int, "Husband with Job", .false. )
         call csv_write( unit_int, "Wife with Job", .false. )
         call csv_write( unit_int, "Both without Job", .true. )

         do s = 1, n_samples_in
            call csv_write( unit_int, sample_labels_in(s), .false. )
            call csv_write( unit_int, job_freq_matrix(:,s), .true. )

         end do

      close( unit_int )

   end if !for root process

end subroutine job_freq_write




end module sim_path_mod
















